import pytest
import quasardb


def test_perf_disable(qdbd_connection):
    qdbd_connection.perf().disable()


def test_perf_enable(qdbd_connection):
    qdbd_connection.perf().enable()

def test_perf_enable_secure_conn(qdbd_secure_connection):
    qdbd_secure_connection.perf().enable()

def test_perf_get__when_empty(qdbd_connection):
    profiles = qdbd_connection.perf().get()
    assert len(profiles) == 0


def test_perf_get_after_blob_put(qdbd_connection, entry_name, random_string):
    qdbd_connection.perf().enable()
    qdbd_connection.blob(entry_name).put(random_string)
    qdbd_connection.blob(entry_name).get()

    profiles = qdbd_connection.perf().get()
    assert len(profiles) == 2
    assert profiles[0][0] == "blob.put"
    assert profiles[1][0] == "common.get"


def test_perf_clear_after_blob_put(qdbd_connection, entry_name, random_string):
    qdbd_connection.perf().enable()
    qdbd_connection.blob(entry_name).put(random_string)

    profiles = qdbd_connection.perf().get()
    assert len(profiles) != 0

    qdbd_connection.perf().clear()
    profiles = qdbd_connection.perf().get()
    assert len(profiles) == 0



import re
import quasardb
import logging

logger = logging.getLogger('quasardb.stats')


stats_prefix = '$qdb.statistics.'
user_pattern = re.compile('\$qdb.statistics.(.*).uid_([0-9]+)$')
total_pattern = re.compile('\$qdb.statistics.(.*)$(?<!uid_[0-9])')

def by_node(conn):
    """
    Returns statistic grouped by node URI.

    Parameters:
    conn: quasardb.Cluster
      Active connection to the QuasarDB cluster
    """
    return {x: of_node(conn, x) for x in conn.endpoints()}

def of_node(conn, uri):
    """
    Returns statistic for a single node.

    Parameters:
    conn: quasardb.Cluster
      Active connection to the QuasarDB cluster

    uri: str
      URI of a node in the cluster, e.g. '127.0.0.1:2836'.
    """
    dconn = conn.node(uri)

    raw = {k: _get_stat(dconn, k) for k in dconn.prefix_get(stats_prefix, 1000)}
    return {'by_uid': _by_uid(raw),
            'cumulative': _cumulative(raw)}

def _get_stat(dconn, k):
    # Ugly, but works: try to retrieve as integer, if not an int, retrieve as blob
    try:
        return dconn.integer(k).get()
    except quasardb.quasardb.Error:
        return str(dconn.blob(k).get(), 'utf-8')

def _by_uid(stats):
    xs = {}
    for k,v in stats.items():
        matches = user_pattern.match(k)
        if matches:
            (metric, uid_str) = matches.groups()
            uid = int(uid_str)
            if not uid in xs:
                xs[uid] = {}

            xs[uid][metric] = v

    return xs

def _cumulative(stats):
    xs = {}

    for k,v in stats.items():
        matches = total_pattern.match(k)
        if matches:
            metric = matches.groups()[0]
            xs[metric] = v

    return xs



from time import sleep
import re
import pytest
import quasardb
import quasardb.stats as qdbst

import test_batch_inserter as batchlib
import conftest

_has_stats = False

def _write_data(conn, table):
    inserter = conn.inserter(
        batchlib._make_inserter_info(table))

    doubles, blobs, strings, integers, timestamps = batchlib._test_with_table(
        inserter,
        table,
        conftest.create_many_intervals(),
        batchlib._regular_push)

def _ensure_stats(conn, table):
    global _has_stats

    # This function is merely here to generate some activity on the cluster,
    # so that we're guaranteed to have some statistics
    if _has_stats is False:
        _write_data(conn,table)
        # Statistics refresh interval is 5 seconds.
        sleep(6)
        _has_stats = True

# A fairly conservative set of user-specific stats we always expect after doing our
# _write_data() activity generating.
#
# We can add keys here if we want to better test against regressions, but for now it's
# fairly conservative.
_expected_user_stats = ['requests.total_count',
                        'requests.successes_count',
                        'requests.bytes_in',
                        'requests.bytes_out',
                        'perf.ts.table_insert.deserialization.total_ns',
                        'perf.ts.table_insert.processing.total_ns']

# Same, but cumulative stats.
_expected_cumulative_stats = ['requests.total_count',
                              'requests.successes_count',
                              'requests.bytes_in',
                              'requests.bytes_out',
                              'perf.ts.buffered_table_insert.deserialization.total_ns',
                              'perf.ts.buffered_table_insert.processing.total_ns',
                              'partitions_count']


def test_stats_by_node (qdbd_secure_connection,
                        secure_table):
    assert(len(_expected_cumulative_stats) > len(_expected_user_stats))
    _ensure_stats(qdbd_secure_connection, secure_table)
    xs = qdbst.by_node(qdbd_secure_connection)

    assert len(xs) == 1

    for _,stats in xs.items():
        assert 'by_uid' in stats
        assert 'cumulative' in stats

        # Test user stats
        for uid,xs in stats['by_uid'].items():
            assert isinstance(uid, int)

            for expected in _expected_user_stats:
                assert expected in xs

            for k,v in xs.items():
                # As far as I know, per-user statistics should *always* be integers
                assert isinstance(v, int)

        # Test cumulative stats
        xs = stats['cumulative']
        for expected in _expected_cumulative_stats:
            assert expected in xs



# pylint: disable=C0103,C0111,C0302,W0212
import unittest
import pytest

import quasardb

def test_connect_with_trace():
    with pytest.raises(quasardb.Error, match=r"at qdb_connect"):
        quasardb.Cluster(uri='invalid_uri')

def test_query_with_trace(qdbd_connection, table, column_name):
    with pytest.raises(quasardb.Error, match=r"at qdb_query"):
        qdbd_connection.query("select notexists from " + table.get_name() + " in range (1990, +40y)")
